unit GFSProxyClient;

interface

uses
  SysUtils, 
  OpenApiRest, 
  OpenApiUtils, 
  GFSProxyJson, 
  GFSProxyDtos;

type
  TRestService = class;
  TService = class;
  TGFSProxyConfig = class;
  TGFSProxyClient = class;
  
  TRestService = class(TCustomRestService)
  protected
    function CreateConverter: TCustomJsonConverter; override;
    function Converter: TJsonConverter;
  end;
  
  IService = interface(IInvokable)
    ['{2DDD33F0-92B5-44DE-899F-4CBCA5CBFE14}']
    function CreateLabel(Body: TCreateLabelRequest): TCreateLabelResponse;
  end;
  
  TService = class(TRestService, IService)
  public
    function CreateLabel(Body: TCreateLabelRequest): TCreateLabelResponse;
  end;
  
  TGFSProxyConfig = class(TCustomRestConfig)
  public
    constructor Create;
  end;
  
  IGFSProxyClient = interface(IRestClient)
    function : IService;
  end;
  
  TGFSProxyClient = class(TCustomRestClient, IGFSProxyClient)
  public
    function : IService;
    constructor Create;
  end;
  
implementation

{ TRestService }

function TRestService.CreateConverter: TCustomJsonConverter;
begin
  Result := TJsonConverter.Create;
end;

function TRestService.Converter: TJsonConverter;
begin
  Result := TJsonConverter(inherited Converter);
end;

{ TService }

function TService.CreateLabel(Body: TCreateLabelRequest): TCreateLabelResponse;
var
  Request: IRestRequest;
  Response: IRestResponse;
begin
  Request := CreateRequest('/shipping/createLabel', 'POST');
  Request.AddBody(Converter.TCreateLabelRequestToJson(Body));
  Request.AddHeader('Content-Type', 'application/json');
  Request.AddHeader('Accept', 'application/json');
  Response := Request.Execute;
  CheckError(Response);
  Result := Converter.TCreateLabelResponseFromJson(Response.ContentAsString);
end;

{ TGFSProxyConfig }

constructor TGFSProxyConfig.Create;
begin
  inherited Create;
  BaseUrl := 'https://dpd-eu.gfsdeliver.com/';
end;

{ TGFSProxyClient }

function TGFSProxyClient.: IService;
begin
  Result := TService.Create(Config);
end;

constructor TGFSProxyClient.Create;
begin
  inherited Create(TGFSProxyConfig.Create);
end;

end.
